from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.core.cache import caches

class Command(BaseCommand):

    """
Ping one or more caches. This is done by performing a non-destructive
access of the cache through Django's cache framework. Some cache
backends can be configured to hide accessibility problems
(e.g. ``django_redis`` with the ``IGNORE_EXCEPTIONS`` turned on). In
such cases, this command will report that the cache is accessible (the
ping will succeed) even if it may not be in fact accessible.
    """
    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument("cache_names", nargs="*")
        parser.add_argument("--all",
                            action="store_true",
                            dest="all",
                            default=False,
                            help="Ping all caches.")

    def handle(self, *args, **options):
        cache_names = options["cache_names"]
        if options["all"]:
            if cache_names:
                raise CommandError("cannot use --all with a cache name")

            cache_names = sorted(settings.CACHES.keys())
        else:
            if not cache_names:
                raise CommandError("specify at least one cache to ping")

            # Make sure all names given exist
            for name in cache_names:
                caches[name]

        failed = False
        for name in cache_names:
            cache = caches[name]
            try:
                "foo" in cache
            except:  # pylint: disable=bare-except
                result = "... unsuccessful"
                failed = True
            else:
                result = "... successful"
            self.stdout.write("Pinging " + name + result)

        if failed:
            raise CommandError("ping failed")



import os

from setuptools import setup, find_packages

version = open('django_cache_management/VERSION').read().strip()
long_description = open("README.rst").read()

try:
    import sphinx
    from sphinx.setup_command import BuildDoc
    import sphinx.apidoc

    class Sphinx(BuildDoc):

        def run(self):
            src_dir = (self.distribution.package_dir or {'': ''})['']
            src_dir = os.path.join(os.getcwd(), src_dir)
            sphinx.apidoc.main(
                ['', '-f', '-o', os.path.join(self.source_dir, '_apidoc'),
                 src_dir])
            BuildDoc.run(self)
except ImportError:
    sphinx = None

setup(
    name="django-cache-management",
    version=version,
    packages=find_packages(),
    author="Louis-Dominique Dubeau",
    author_email="ldd@lddubeau.com",
    description="Cache management for Django.",
    long_description=long_description,
    license="MPL 2.0",
    keywords=["Django", "caching"],
    url="https://github.com/lddubeau/django-cache-management",
    install_requires=[
        'Django>=1.11',
        'six'
    ],
    classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: POSIX",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
    setup_requires=['sphinx'],
    include_package_data=True,
    cmdclass={'sphinx': Sphinx} if sphinx is not None else {}
)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.core.cache import caches

class Command(BaseCommand):

    """
Clear one or more caches.

Note that the clear operation is not guaranteed to be atomic.
    """
    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument("cache_names", nargs="*")
        parser.add_argument('--noop',
                            action='store_true',
                            dest='noop',
                            default=False,
                            help='Do not actually perform the action.')
        parser.add_argument('--all',
                            action='store_true',
                            dest='all',
                            default=False,
                            help='Clear all caches.')
        parser.add_argument('--method',
                            dest='method',
                            default="conservative",
                            choices=("conservative", "django-clear"),
                            help="""
The method to use. The method "conservative" will remove only the
keys that begin with the cache's prefix. The method "django-clear"
will use Django's ``clear`` method, which will delete even keys that are
not prefixed with the cache's key. "conservative" is the default.
""".strip())

    def handle(self, *args, **options):
        cache_names = options["cache_names"]
        if options["all"]:
            if cache_names:
                raise CommandError("cannot use --all with a cache name")

            cache_names = settings.CACHES.keys()
        else:
            if not cache_names:
                raise CommandError("specify at least one cache to clear")

            # Make sure all names given exist
            for name in cache_names:
                caches[name]

        method = options["method"]
        noop = options["noop"]

        action = "Clearing " if not noop else "Not clearing "

        if method == "conservative":
            for name in cache_names:
                config = settings.CACHES[name]
                backend = config["BACKEND"]
                failed = False
                if backend.startswith("django_redis."):
                    try:
                        import django_redis
                        django_redis.get_redis_connection(name)
                        # Yes, we grab all exceptions. It is up to the
                        # user to diagnose how their configuration is
                        # wrong.
                    except:  # pylint: disable=bare-except
                        failed = True
                else:
                    failed = True

                if failed:
                    raise CommandError(
                        "clearcache does not know how to "
                        "conservatively clear a "
                        "cache with backend {0}".format(backend))

            for name in cache_names:
                self.stdout.write(action + name)
                config = settings.CACHES[name]
                cache = caches[name]
                prefix = cache.key_prefix
                backend = config["BACKEND"]
                if backend.startswith("django_redis."):
                    import django_redis
                    con = django_redis.get_redis_connection(name)
                    keys = con.keys(prefix + ':*')
                    if keys and not noop:
                        con.delete(*keys)
        else:
            for name in cache_names:
                self.stdout.write(action + name)
                cache = caches[name]
                if not noop:
                    cache.clear()
